@@ -764,4 +764,6 @@ bool OT_exists
 #define OT_PUSH(ptr,stk)                OT_T_INS(stk,ABS_CAST(OT_Obj,ptr))
 #define OT_POP(stk)                     OT_DEL_E(stk,OT_cnt(stk)-1)
 #define OT_TOP(type,stk)                OT_GET(type,stk,OT_cnt(stk)-1)
+#define OT_TOP_small(type,stk)          ( (type)((long)OT_get(stk,OT_cnt(stk)-1)) )
+#define OT_TOP_short(stk)               ( (short)((long)OT_get(stk,OT_cnt(stk)-1)) )
 #define OT_DELSTK(stk)                  OT_DEL_T(stk)
