@@ -143,8 +143,8 @@ ConcreteType( Scn_Stream_IMP )
   ROW(wc_char) cBackBuf; /* lookahead back buffer */
   MAP(int,_)   cCSkip;   /* characters to skip */
   /* Scanner */
-  scn_t      cScanner; /* Following variables are unpacked from cScanner */
   short      Group;    /* Group index or -1                              */
+  scn_t      cScanner; /* Following variables are unpacked from cScanner */
   long      *cStaEdg;  /* Note that this array will be offset in Scn_bgn */
   short     *cStaFin;
   wc_char   *cEdgeC;
@@ -689,7 +689,7 @@ static symbol Stream_init_Pattern_sym(Scn_Stream_IMP s, c_string p, c_bool seq)
     buf[1] = p[i+1];
     buf[2] = p[i+2];
     buf[3] = '\0';
-    c1 = atoi(buf);
+    c1 = atoi((const char*)buf);
     if( i == 0 )
     {
       if( seq )
@@ -712,7 +712,7 @@ static symbol Stream_init_Pattern_sym(Scn_Stream_IMP s, c_string p, c_bool seq)
       buf[1] = p[i+1];
       buf[2] = p[i+2];
       buf[3] = '\0';
-      c2 = atoi(buf);
+      c2 = atoi((const char*)buf);
       while( ++c1 <= c2 )
       {
         Sink_printf(snk,"%c",(c_byte)c1);
