@@ -16,6 +16,7 @@
 #include "scn_imp.h"
 #include "pathes.h"
 #include "sink.h"
+#include "line_scn.h"
 
 /*!!
    Beside copying and disposing of scanner definitions 
@@ -340,7 +341,7 @@ static void Scn_C_SourceScanner(FILE* fp, Scn_T x)
       fprintf(fp,"  /* %3d */", i);
       for (j = 0; j < ed; j++)
       { long bgn = scn->StaEdg[i];
-        fprintf(fp,"  %9ld",scn->EdgeC[bgn+j]);
+        fprintf(fp,"  %9ld",(long int)scn->EdgeC[bgn+j]);
         if (             bgn+j+1 < SCN_Edges(scn)) fprintf(fp,",");
         if (j%6 == 5 && bgn+j+1 < SCN_Edges(scn) && j+1 < ed)
           fprintf(fp,"\n           ");
@@ -524,53 +525,57 @@ void Scn_C_Modul(c_string Environ, Scn_T x, c_bool shortform)
 }
 
 
-static void Scn_Src_SourceScanner(FILE* fp, Scn_T x)
+static void Scn_Src_SourceScanner
+            (
+              FILE* fp, Scn_T x, c_string prefix, c_string suffix
+            )
 /* writes source for ( group ) scanner definition 'x' to file 'fp'
 */
 { int i; scn_t scn = Scn_IMP(x);
-  fprintf(fp,"Name=%s\n",scn->Name);
+  c_string nl = ( !STR_EMPTY(prefix) && !STR_EMPTY(suffix) ) ? "\\n" : "";
+  fprintf(fp,"%sName=%s%s%s\n",prefix,scn->Name,nl,suffix);
   if( scn->Groups == 0 )
   {
-    fprintf(fp,"StaEdg\n");
+    fprintf(fp,"%sStaEdg%s%s\n",prefix,nl,suffix);
     for (i = 0; i < scn->States+1; i++)
     {
-      fprintf(fp,"%ld\n",scn->StaEdg[i]);
+      fprintf(fp,"%s%ld%s%s\n",prefix,scn->StaEdg[i],nl,suffix);
     }
-    fprintf(fp,"StaFin\n");
+    fprintf(fp,"%sStaFin%s%s\n",prefix,nl,suffix);
     for (i = 0; i < scn->States; i++)
     {
-      fprintf(fp,"%d\n",scn->StaFin[i]);
+      fprintf(fp,"%s%d%s%s\n",prefix,scn->StaFin[i],nl,suffix);
     }
-    fprintf(fp,"EdgeC\n");
+    fprintf(fp,"%sEdgeC%s%s\n",prefix,nl,suffix);
     for (i = 0; i < scn->States; i++)
     { long j, ed = scn->StaEdg[i+1] - scn->StaEdg[i];
       for (j = 0; j < ed; j++)
       { long bgn = scn->StaEdg[i];
-        fprintf(fp,"%ld\n",scn->EdgeC[bgn+j]);
+        fprintf(fp,"%s%ld%s%s\n",prefix,(long int)scn->EdgeC[bgn+j],nl,suffix);
       }
     }
-    fprintf(fp,"EdgeS\n");
+    fprintf(fp,"%sEdgeS%s%s\n",prefix,nl,suffix);
     for (i = 0; i < scn->States; i++)
     { long j, ed = scn->StaEdg[i+1] - scn->StaEdg[i];
       for (j = 0; j < ed; j++)
       { long bgn = scn->StaEdg[i];
-        fprintf(fp,"%ld\n",scn->EdgeS[bgn+j]);
+        fprintf(fp,"%s%ld%s%s\n",prefix,scn->EdgeS[bgn+j],nl,suffix);
       }
     }
-    fprintf(fp,"TokId\n");
+    fprintf(fp,"%sTokId%s%s\n",prefix,nl,suffix);
     for (i = 0; i < scn->Tokens; i++)
-      fprintf(fp," %s\n",scn->TokId[i]);
-    fprintf(fp,"Flags\n");
+      fprintf(fp,"%s %s%s%s\n",prefix,scn->TokId[i],nl,suffix);
+    fprintf(fp,"%sFlags%s%s\n",prefix,nl,suffix);
     for (i = 0; i < scn->Tokens; i++)
     {
-      fprintf(fp,"%d\n", (int)scn->Flags[i]);
+      fprintf(fp,"%s%d%s%s\n", prefix,(int)scn->Flags[i],nl,suffix);
     }
     if( scn->Switch != (short*)NULL )
     {
-      fprintf(fp,"Switch\n");
+      fprintf(fp,"%sSwitch%s%s\n",prefix,nl,suffix);
       for (i = 0; i < scn->Tokens; i++)
       {
-        fprintf(fp,"%d\n",scn->Switch[i]);
+        fprintf(fp,"%s%d%s%s\n",prefix,scn->Switch[i],nl,suffix);
       }
     }
   }
@@ -578,44 +583,96 @@ static void Scn_Src_SourceScanner(FILE* fp, Scn_T x)
   {
     for( i=0; i < scn->dyckcnt; ++i )
     {
-      fprintf(fp,"# dyck scanner '%s'\n\nDyck\n",scn->dyckpat[i]->Name);
-      Scn_Src_SourceScanner(fp,scn->dyckpat[i]);
+      fprintf
+      (
+        fp,"%s# dyck scanner '%s'%s%s%s\n\n%sDyck%s%s\n",
+        prefix,scn->dyckpat[i]->Name,nl,nl,suffix,prefix,nl,suffix
+      );
+      Scn_Src_SourceScanner(fp,scn->dyckpat[i],prefix,suffix);
     }
-    fprintf(fp,"DyckToken\n");
+    fprintf(fp,"%sDyckToken%s%s\n",prefix,nl,suffix);
     for( i=0; i < scn->dyckcnt; ++i )
     {
-      fprintf(fp,"%d\n",scn->dyckidx[i]);
+      fprintf(fp,"%s%d%s%s\n",prefix,scn->dyckidx[i],nl,suffix);
     }
   }
 }
 
 
-void Scn_Src_Source(FILE* fp, c_string fid, Scn_T x)
+void Scn_Src_Source
+     (
+       FILE* fp, c_string fid, Scn_T x, c_string prefix, c_string suffix
+     )
 /* writes source for scanner ( group ) definition 'x' to file 'fp';
-   produces a source header if filename 'fid' is not empty
+   produces a source header if filename 'fid' is not empty;
+   encapsulates output lines with 'prefix' and 'suffix'
 */
 { int i; scn_t scn = Scn_IMP(x);
+  c_string nl = ( !STR_EMPTY(prefix) && !STR_EMPTY(suffix) ) ? "\\n" : "";
   if( !STR_EMPTY(fid) )
-    fprintf(fp,"# [%s] Scanner table for '%s'\n\n", fid, scn->Name);
+    fprintf
+    (
+      fp,"%s# [%s] Scanner table for '%s'%s%s%s\n\n",
+      prefix,fid,scn->Name,nl,nl,suffix
+    );
   for( i=0; i < scn->Groups; ++i )
   {
-    fprintf(fp,"# group scanner '%s'\n\nGroup\n",scn->GrpScn[i]->Name);
-    Scn_Src_SourceScanner(fp,scn->GrpScn[i]);
+    fprintf
+    (
+      fp,"%s# group scanner '%s'%s%s%s\n\n%sGroup%s%s\n",
+      prefix,scn->GrpScn[i]->Name,nl,nl,suffix,prefix,nl,suffix
+    );
+    Scn_Src_SourceScanner(fp,scn->GrpScn[i],prefix,suffix);
   }
-  fprintf(fp,"# scanner group '%s'\n\nLanguage\n",scn->Name);
-  Scn_Src_SourceScanner(fp, x);
-  fprintf(fp,"\n");
+  fprintf
+  (
+    fp,"%s# scanner group '%s'%s%s%s\n\n%sLanguage%s%s\n",
+    prefix,scn->Name,nl,nl,suffix,prefix,nl,suffix
+  );
+  Scn_Src_SourceScanner(fp,x,prefix,suffix);
+  fprintf(fp,"%s%s%s\n",prefix,nl,suffix);
 }
 
-void Scn_Src_Modul(c_string Environ, Scn_T x)
+void Scn_Src_Modul(c_string Environ, Scn_T x, string Template)
 /* creates source [$'Environ'/]<Language>.ltbl
-   for scanner ( group ) definition 'x'
+   for scanner ( group ) definition 'x';
+   it uses the template file for the export, if 'Template' is a regular file 
 */
-{ FILE *tf; c_string fid;
+{ FILE *tf = NULL; c_string fid;
   scn_t scn = Scn_IMP(x);
   fid = Str_printf("%s.ltbl",scn->Name);
   tf = OpenPath(Environ,fid,"","wt");
-  Scn_Src_Source(tf,fid,x);
+  if( STR_EMPTY(Template) ) Scn_Src_Source(tf,fid,x,"","");
+  else
+  { FILE* pf = NULL; c_string line;
+    pf = OpnFile(Template,"rt");
+    while( (line=Line_get(pf)) != (c_string)NULL )
+    { c_string prefix, suffix;
+      if( Line_split(line,"@TABLE@",&prefix,&suffix) )
+      {
+        fprintf(tf,"%sltbl%s%s\n",prefix?prefix:"",scn->Name,suffix?suffix:"");
+      }
+      else
+      if( Line_split(line,"@LINE@",&prefix,&suffix) )
+      {
+        if( prefix != (c_string)NULL && suffix != (c_string)NULL )
+          Scn_Src_Source(tf,fid,x,prefix,suffix);
+        else
+        if( prefix != (c_string)NULL )
+          Scn_Src_Source(tf,fid,x,prefix,"");
+        else
+        if( suffix != (c_string)NULL )
+          Scn_Src_Source(tf,fid,x,"",suffix);
+        else
+          Scn_Src_Source(tf,fid,x,"","");
+      }
+      else fprintf(tf,"%s\n",line);
+      if( prefix != (c_string)NULL ) FreeMem(prefix);
+      if( suffix != (c_string)NULL ) FreeMem(suffix);
+      FreeMem(line);
+    }
+    fclose(pf);
+  }
   fclose(tf);
   FreeMem(fid);
 }
