@@ -177,7 +177,8 @@ void CTX_usage(CTX_T ctx)
         case CTX_STRING : fprintf(STDERR,"=string"); break;
         case CTX_NAME   : fprintf(STDERR,"=name"); break;
         case CTX_INT    : fprintf(STDERR,"=int"); break;
-        default         : fprintf(STDERR,"???(%d)",ctx_imp->tbl[i].cat); break;
+        default         : 
+          fprintf(STDERR,"%c%c%c(%d)",'?','?','?',ctx_imp->tbl[i].cat); break;
       }
       fprintf(STDERR,"]");
     }
@@ -299,7 +300,8 @@ void CTX_list(CTX_T ctx)
       case CTX_ARG : fprintf(STDOUT,"Arg"); break;
       case CTX_OPT : fprintf(STDOUT,"Opt"); break;
       case CTX_ENV : fprintf(STDOUT,"Env"); break;
-      default      : fprintf(STDOUT,"???(%d)",ctx_imp->tbl[i].cat); break;
+      default      : 
+        fprintf(STDOUT,"%c%c%c(%d)",'?','?','?',ctx_imp->tbl[i].cat); break;
     }
     fprintf(STDOUT," %-*s : ",maxlen,ctx_imp->tbl[i].ide);
     switch( ctx_imp->tbl[i].typ )
@@ -309,7 +311,8 @@ void CTX_list(CTX_T ctx)
       case CTX_STRING : fprintf(STDOUT,"string"); break;
       case CTX_NAME   : fprintf(STDOUT,"name  "); break;
       case CTX_INT    : fprintf(STDOUT,"int   "); break;
-      default         : fprintf(STDOUT,"???(%d)",ctx_imp->tbl[i].cat); break;
+      default         : 
+        fprintf(STDOUT,"%c%c%c(%d)",'?','?','?',ctx_imp->tbl[i].cat); break;
     }
     fprintf(STDOUT," = ");
     if (ctx_imp->tbl[i].asg)
