@@ -27,7 +27,7 @@
 #include "styd_cim.h"
 
 
-/* ------------------------- Macros, Types & Globals -------------------- */
+/* ------------------------- Macros, Types & Globals ---------------------- */
 
 #define infile            CTX_EVAL(c_string,"InFile")
 #define outfile           CTX_EVAL(c_string,"OutFile")
@@ -76,6 +76,7 @@ ConcreteType(StyDoc_T)
   MAP(symbol,styxQlxDfn)   pGMap;    // group symbol |--> definition
   MAP(symbol,styxQlxDfn)   pTMap;    // token symbol |--> definition
   MAP(symbol,long)         pNTMap;   // nonterminal symbol |--> index 
+  MAP(symbol,symbol)       pClsMap;  // nonterminal symbol |--> class symbol
   SET(symbol)              pLSet;    // embedded languages
   ROW(_)                   pDocBase; // language documentation base:
                                      // ROW( T/NT symbol, 
@@ -94,7 +95,10 @@ static SET(symbol)             pClsToSplit;
 static SET(symbol)             pIgnRecRef;
 static MAP(symbol,SET(symbol)) pExcludedProds;
 
-/* ------------------------- Auxiliary ---------------------------------- */
+static c_bool                  bConcreteCfg = C_False;
+
+
+/* ------------------------- Auxiliary ------------------------------------ */
 
 static c_string SD_com_string
                 (
@@ -248,17 +252,20 @@ static c_bool SD_IsNTMetaSym(symbol pNTSym, symbol* ppMetaSym)
 }
 
 static c_bool SD_IsTMetaSym(symbol pTSym)
-// Meta-Symbols: ()|+{}[]- and ..
-{ c_string szMetaSet = "(){}[]|+-",
+// Meta-Symbols: ()|+{}[]<>-/, and ..
+{ c_string szMetaSet = "(){}[]<>|+-/,",
            szTSym    = symbolToString(pTSym);
   if( strlen(szTSym) == 1 )
     return strstr(szMetaSet,szTSym) != (c_string)NULL;
   if( strlen(szTSym) == 2 )
-    return !strcmp(szTSym,"..");
+    return !strcmp(szTSym,"..") ||
+           !strcmp(szTSym,">=") ||
+           !strcmp(szTSym,"<=") ||
+           !strcmp(szTSym,"<?");
   return C_False;
 }
 
-/* ------------------------- Init & Quit -------------------------------- */
+/* ------------------------- Init & Quit ---------------------------------- */
 
 static void SD_init(int argc, c_string argv[])
 /* init STYX documentation generation */
@@ -269,15 +276,10 @@ static void SD_init(int argc, c_string argv[])
   MAP_init();
   styx_initSymbols();
   HP_init();
-  pNTsToKeep = HS_CREATE_ADTSET(symbol);
-  if( strlen(nts_to_keep) > 0 )
-  {
-    pItr = PIT_make(nts_to_keep); 
-    while( ( szS = PIT_read(pItr) ) != NULL )
-      HS_SET_ELM(stringToSymbol(szS),pNTsToKeep);
-    PIT_drop(pItr);
-  }
   pClsToSplit = HS_CREATE_ADTSET(symbol);
+  if( !strcmp(cls_to_split,"--all--") )
+    bConcreteCfg = C_True;
+  else
   if( strlen(cls_to_split) > 0 )
   {
     pItr = PIT_make(cls_to_split); 
@@ -285,8 +287,16 @@ static void SD_init(int argc, c_string argv[])
       HS_SET_ELM(stringToSymbol(szS),pClsToSplit);
     PIT_drop(pItr);
   }
+  pNTsToKeep = HS_CREATE_ADTSET(symbol);
+  if( !bConcreteCfg && strlen(nts_to_keep) > 0 )
+  {
+    pItr = PIT_make(nts_to_keep); 
+    while( ( szS = PIT_read(pItr) ) != NULL )
+      HS_SET_ELM(stringToSymbol(szS),pNTsToKeep);
+    PIT_drop(pItr);
+  }
   pIgnRecRef = HS_CREATE_ADTSET(symbol);
-  if( strlen(ign_rref) > 0 )
+  if( !bConcreteCfg && strlen(ign_rref) > 0 )
   {
     pItr = PIT_make(ign_rref); 
     while( ( szS = PIT_read(pItr) ) != NULL )
@@ -352,6 +362,7 @@ static StyDoc_T SD_init_base(void)
   pStyDoc->pGMap       = MAP_newPrimMap();
   pStyDoc->pTMap       = MAP_newPrimMap();
   pStyDoc->pNTMap      = MAP_newPrimMap();
+  pStyDoc->pClsMap     = MAP_newPrimMap();
   pStyDoc->pLSet       = HS_CREATE_ADTSET(symbol);
   pStyDoc->pDocBase    = OT_CREATE_ADT(_);
   pStyDoc->pHPat       = (HPat_T)NULL;
@@ -363,7 +374,7 @@ static StyDoc_T SD_init_base(void)
   return pStyDoc;
 }
 
-/* ------------------------- Output Generation -------------------------- */
+/* ------------------------- Output Generation ---------------------------- */
 
 static void SD_gendoc_raw(StyDoc_T pStyDoc)
 /* generate raw documentation - without template */
@@ -380,6 +391,9 @@ static void SD_gendoc_raw(StyDoc_T pStyDoc)
                                 symbol,pStyDoc->pDocBase,pStyDoc->lDocIdx*2
                               ),
                               pMbrSym;
+    if( lI >= MAP_count(pStyDoc->pTMap) &&
+        MAP_defined(pStyDoc->pClsMap,pSym) )
+      pSym = MAP_apply(symbol,pStyDoc->pClsMap,pSym);
     if( lI == 0 ) 
       fprintf(pStyDoc->pSysOut,"\nToken Section\n");
     else 
@@ -392,6 +406,9 @@ static void SD_gendoc_raw(StyDoc_T pStyDoc)
       for( lK=0; lK < OT_CNT(pCurRow); ++lK )
       {
         pMbrSym = OT_GET(symbol,pCurRow,lK);
+        if( lI >= MAP_count(pStyDoc->pTMap) &&
+            MAP_defined(pStyDoc->pClsMap,pMbrSym) )
+          pMbrSym = MAP_apply(symbol,pStyDoc->pClsMap,pMbrSym);
         fprintf(pStyDoc->pSysOut," %s",symbolToString(pMbrSym));
       }
     }
@@ -410,6 +427,11 @@ static c_bool SD_gendoc_dref(HPat_T pHPat)
   pStyDoc->lComIdx = 0;
   if( pStyDoc->lDocIdx >= 0 && pStyDoc->lDocIdx < OT_CNT(pStyDoc->pDocBase)/2 )
   { symbol pSym = OT_GET(symbol,pStyDoc->pDocBase,pStyDoc->lDocIdx*2);
+    if( pStyDoc->lDocIdx >= MAP_count(pStyDoc->pTMap) )
+    {
+      if( MAP_defined(pStyDoc->pClsMap,pSym) )
+        pSym = MAP_apply(symbol,pStyDoc->pClsMap,pSym);
+    }
     fprintf(pStyDoc->pSysOut,"%s",symbolToString(pSym));
     return C_True;
   }
@@ -440,6 +462,11 @@ static c_bool SD_gendoc_did(HPat_T pHPat)
           Sink_printf(pSnk,":%s",GLS_Tok_string(pDId));
         }
         else
+        if( styxQlxGrp_igrp(pDGrp) )
+        {
+          Sink_printf(pSnk,":[initial]");
+        }
+        else
         if( styxQlxGrp_pgrp(pDGrp,&pDId) )
         {
           Sink_printf(pSnk,":[push], %s",GLS_Tok_string(pDId));
@@ -460,7 +487,11 @@ static c_bool SD_gendoc_did(HPat_T pHPat)
       }
     }
     else
+    {
+      if( MAP_defined(pStyDoc->pClsMap,pSym) )
+        pSym = MAP_apply(symbol,pStyDoc->pClsMap,pSym);
       fprintf(pStyDoc->pSysOut,"%s",symbolToString(pSym));
+    }
     return C_True;
   }
   return C_False;
@@ -645,7 +676,13 @@ static c_bool SD_gendoc_mid(HPat_T pHPat)
   { symbol pMbrSym = OT_GET(symbol,pStyDoc->pCurRow,pStyDoc->lMbrIdx),
            pDocSym = pMbrSym;
     if( pStyDoc->lDocIdx >= MAP_count(pStyDoc->pTMap) )
-      SD_IsNTMetaSym(pMbrSym,&pDocSym);
+    {
+      if( ! SD_IsNTMetaSym(pMbrSym,&pDocSym) )
+      {
+        if( MAP_defined(pStyDoc->pClsMap,pDocSym) )
+          pDocSym = MAP_apply(symbol,pStyDoc->pClsMap,pDocSym);
+      }
+    }
     if( !HP_fun_defined(pHPat,SD_PAT_MBRID) )
       fprintf(pStyDoc->pSysOut,"%s",symbolToString(pDocSym));
     else
@@ -800,7 +837,7 @@ static c_bool SD_gendoc_cfg(HPat_T pHPat)
   return C_True;
 }
 
-/* ------------------------- Load & Unload ------------------------------ */
+/* ------------------------- Load & Unload -------------------------------- */
 
 #define IS_QEXP_UDC(e) (    styxExp_union(e,_,_)  \
                          || styxExp_diff(e,_,_)   \
@@ -1185,6 +1222,15 @@ static void SD_load_lex(GLS_Lst(styxQlxDfn) pQDfns, StyDoc_T pStyDoc)
                   StyDoc_T pStyDoc
                 );
 
+static c_bool SD_splitClass(StyDoc_T pStyDoc, long lClsIdx)
+/* create separate rules for each nonterminal */
+{ symbol pClsSym = stringToSymbol(PLR_symName
+                   (
+                     pStyDoc->pTab, PLR_tokenCnt(pStyDoc->pTab) + lClsIdx
+                   ));
+  return HS_MBR_ELM(pClsSym,pClsToSplit);
+}
+
 static ROW(symbol) SD_load_ntprod
                    (
                      long lNTIdx, long lPrdIdx, long lDocIdx, c_bool bNTList,
@@ -1192,12 +1238,12 @@ static ROW(symbol) SD_load_ntprod
                    )
 /* load documentation base for a production of nonterminal 'lNTIdx' */
 { long             lMbrCnt, lSym, lSymIdx, lClsIdx, lTyp, 
-                   lSubSym, lSubIdx, lI, lJ;
+                   lSubSym, lClsSym, lSubIdx, lI, lJ;
   ROW(ROW(symbol)) pNTLines = MAP_apply(ROW(ROW(symbol)),pEBNF,lDocIdx),
                               pMbrLines;
   ROW(symbol)      pNTLine  = OT_CREATE_ADT(symbol), pMbrLine;
-  c_string         szSym;
-  symbol           pSym;
+  c_string         szSym, szClsSym;
+  symbol           pSym, pClsSym;
   c_bool           bOptNT;
   lMbrCnt = PLR_prodSymCnt(pStyDoc->pTab,lPrdIdx);
   for( lI=0; lI < lMbrCnt; ++lI )
@@ -1210,11 +1256,25 @@ static ROW(symbol) SD_load_ntprod
       lClsIdx   = PLR_ntClassId(pStyDoc->pTab,lSymIdx);
       lSubIdx   = SD_load_ntgroup(lSymIdx,pEBNF,pStyDoc);
       lSubSym   = lSubIdx + PLR_tokenCnt(pStyDoc->pTab);
+      lClsSym   = lClsIdx + PLR_tokenCnt(pStyDoc->pTab);
       szSym     = PLR_symName(pStyDoc->pTab,lSubSym);
+      szClsSym  = SD_splitClass(pStyDoc,lClsIdx)
+                  ? szSym : PLR_symName(pStyDoc->pTab,lClsSym);
       pSym      = stringToSymbol(szSym);
+      pClsSym   = stringToSymbol(szClsSym);
       pMbrLines = MAP_apply(ROW(ROW(symbol)),pEBNF,lSubIdx);
       bOptNT    =   ACFG_HAS_NTTYP(pStyDoc->aPol[lSubIdx],ACFG_NTTYP_OPT) &&
                   ! ACFG_HAS_NTTYP(pStyDoc->aPol[lSubIdx],ACFG_NTTYP_RECTYP);
+      if( HS_MBR_ELM(pSym,pNTsToKeep) ||
+          ( ( MAP_defined(pStyDoc->pNTMap,pSym) ||
+              SD_IsNTRecRef(lSymIdx, pStyDoc)   ||
+              ! ACFG_HAS_NTTYP(pStyDoc->aPol[lClsIdx],ACFG_NTTYP_LIST) ) &&
+            ( ACFG_HAS_NTTYP(pStyDoc->aPol[lSubIdx],ACFG_NTTYP_RECTYP) ||
+              ( OT_CNT(pMbrLines) > 1 &&
+                ! ( ACFG_HAS_NTTYP(pStyDoc->aPol[lSymIdx],ACFG_NTTYP_REF1) &&
+                    ignProd(PLR_prodName(pStyDoc->pTab,lPrdIdx)) && 
+                    lMbrCnt == 1 ) ) ) ) )
+/*HM 26.1.2005
       if( ( MAP_defined(pStyDoc->pNTMap,pSym) ||
             HS_MBR_ELM(pSym,pNTsToKeep)       ||
             SD_IsNTRecRef(lSymIdx, pStyDoc)   ||
@@ -1224,9 +1284,13 @@ static ROW(symbol) SD_load_ntprod
               ! ( ACFG_HAS_NTTYP(pStyDoc->aPol[lSymIdx],ACFG_NTTYP_REF1) &&
                   ignProd(PLR_prodName(pStyDoc->pTab,lPrdIdx)) && 
                   lMbrCnt == 1 ) ) ) )
+*/
       {
         if( ! MAP_defined(pStyDoc->pNTMap,pSym) )
         {
+          if( pSym != pClsSym &&
+              !MAP_defined(pStyDoc->pNTMap,pClsSym) )
+            MAP_dfndom(pStyDoc->pClsMap,pSym,pClsSym);
           MAP_dfndom(pStyDoc->pNTMap,pSym,lSubIdx);
           OT_PUSH(pSym,pStyDoc->pDocBase);
           OT_PUSH(pMbrLines,pStyDoc->pDocBase);
@@ -1459,15 +1523,6 @@ static void SD_load_ntlist
   else OT_DEL_T(pListLine);
 }
 
-static c_bool SD_splitClass(StyDoc_T pStyDoc, long lClsIdx)
-/* create separate rules for each nonterminal */
-{ symbol pClsSym = stringToSymbol(PLR_symName
-                   (
-                     pStyDoc->pTab, PLR_tokenCnt(pStyDoc->pTab) + lClsIdx
-                   ));
-  return HS_MBR_ELM(pClsSym,pClsToSplit);
-}
-
 static long SD_load_ntgroup
             (
               long lNTIdx, MAP(long,ROW(ROW(symbol))) pEBNF, StyDoc_T pStyDoc
@@ -1488,12 +1543,22 @@ static long SD_load_ntgroup
                                 (
                                   pStyDoc->pTab, 
                                   PLR_tokenCnt(pStyDoc->pTab) + lDocIdx
-                                ));
+                                )),
+                     pClsSym  = SD_splitClass(pStyDoc,lClsIdx)
+                                ? pNTSym
+                                : stringToSymbol(PLR_symName
+                                  (
+                                    pStyDoc->pTab, 
+                                    PLR_tokenCnt(pStyDoc->pTab) + lClsIdx
+                                  ));
     HS_Itr           pItr;
     MAP_dfndom(pEBNF,lDocIdx,pNTLines);
     if( ACFG_HAS_NTTYP(pStyDoc->aPol[lNTIdx],ACFG_NTTYP_START) &&
         ! MAP_defined(pStyDoc->pNTMap,pNTSym)                   )  
     {
+      if( pNTSym != pClsSym &&
+          !MAP_defined(pStyDoc->pNTMap,pClsSym) )
+        MAP_dfndom(pStyDoc->pClsMap,pNTSym,pClsSym);
       MAP_dfndom(pStyDoc->pNTMap,pNTSym,lDocIdx);
       OT_PUSH(pNTSym,pStyDoc->pDocBase);
       OT_PUSH(pNTLines,pStyDoc->pDocBase);
@@ -1556,6 +1621,94 @@ static void SD_load_cfg(StyDoc_T pStyDoc)
   MAP_freeMap(pEBNF);
 }
 
+static void SD_load_cnt
+            (
+              long lNTIdx,
+              MAP(long,ROW(ROW(symbol))) pEBNF, StyDoc_T pStyDoc
+            )
+/* load documentation base for a nonterminal */
+{ long             lPrdIdx, lSymCnt, lSym, lI;
+  ROW(ROW(symbol)) pNTLines = MAP_apply(ROW(ROW(symbol)),pEBNF,lNTIdx);
+  ROW(symbol)      pNTLine;
+  HS_Itr           pItr;
+  symbol           pNTSym   = stringToSymbol(PLR_symName
+                              (
+                                pStyDoc->pTab, 
+                                PLR_tokenCnt(pStyDoc->pTab) + lNTIdx
+                              )), pProdSym;
+  c_bool           bExcluded;
+  HS_FORALL(lPrdIdx,pItr,pStyDoc->aPrd[lNTIdx])
+  { 
+    pProdSym  = stringToSymbol(PLR_prodName(pStyDoc->pTab,lPrdIdx));
+    bExcluded = MAP_defined(pExcludedProds,pNTSym) &&
+                HS_MBR_ELM
+                (
+                  pProdSym,MAP_apply(SET(symbol),pExcludedProds,pNTSym)
+                );
+    if( ! bExcluded )
+    {
+      pNTLine = OT_CREATE_ADT(symbol);
+      OT_PUSH(pNTLine,pNTLines);
+      lSymCnt = PLR_prodSymCnt(pStyDoc->pTab,lPrdIdx);
+      for( lI=0; lI < lSymCnt; ++lI )
+      { c_string szSym;
+        lSym = PLR_prodSymbol(pStyDoc->pTab,lPrdIdx,lI); 
+        szSym = PLR_symName(pStyDoc->pTab,lSym);
+        OT_PUSH(stringToSymbol(szSym),pNTLine);
+      }
+    }
+  }
+}
+
+static void SD_load_ccfg(StyDoc_T pStyDoc)
+/* load documentation base for the concrete context free grammar */
+{ MAP(long,ROW(ROW(symbol))) pEBNF;
+  ROW(ROW(symbol))           pNTLines;
+  MAPIT                      pItr;
+  long                       lCnt = PLR_startCnt(pStyDoc->pTab), 
+                             lDocIdx, lI;
+  symbol                     pNTSym;
+  pEBNF = MAP_newPrimMap();
+  for( lI=0; lI < lCnt; ++lI )
+  {
+    lDocIdx = PLR_startSymbol(pStyDoc->pTab,lI)-PLR_tokenCnt(pStyDoc->pTab);
+    pNTSym  = stringToSymbol
+              (
+                PLR_symName(pStyDoc->pTab,PLR_tokenCnt(pStyDoc->pTab)+lDocIdx)
+              );
+    MAP_dfndom(pStyDoc->pNTMap,pNTSym,lDocIdx);
+    pNTLines = OT_CREATE(ROW(symbol),primCopy,OT_delT,primEqual);
+    MAP_dfndom(pEBNF,lDocIdx,pNTLines);
+    OT_PUSH(pNTSym,pStyDoc->pDocBase);
+    OT_PUSH(pNTLines,pStyDoc->pDocBase);
+    SD_load_cnt(lDocIdx,pEBNF,pStyDoc);
+  }
+  lCnt = PLR_nontermCnt(pStyDoc->pTab);
+  for( lI=0; lI < lCnt; ++lI )
+  {
+    if( !MAP_defined(pEBNF,lI) )
+    {
+      pNTSym  = stringToSymbol
+                (
+                  PLR_symName(pStyDoc->pTab,PLR_tokenCnt(pStyDoc->pTab)+lI)
+                );
+      MAP_dfndom(pStyDoc->pNTMap,pNTSym,lI);
+      pNTLines = OT_CREATE(ROW(symbol),primCopy,OT_delT,primEqual);
+      MAP_dfndom(pEBNF,lI,pNTLines);
+      OT_PUSH(pNTSym,pStyDoc->pDocBase);
+      OT_PUSH(pNTLines,pStyDoc->pDocBase);
+      SD_load_cnt(lI,pEBNF,pStyDoc);
+    }
+  }
+  MAP_forItrAsg(pNTSym,lDocIdx,pItr,pStyDoc->pNTMap)
+  {
+    MAP_rmvdom(pEBNF,lDocIdx);
+  }
+  MAP_forItrAsg(lDocIdx,pNTLines,pItr,pEBNF)
+    if( pNTLines != (ROW(ROW(symbol)))NULL ) OT_DEL_T(pNTLines);
+  MAP_freeMap(pEBNF);
+}
+
 static c_bool SD_load_pat(StyDoc_T pStyDoc)
 /* load template */
 { 
@@ -1638,7 +1791,10 @@ static c_bool SD_load(StyDoc_T* ppStyDoc)
           acfg_init_doc(pStyDoc->pTab,
                         &pStyDoc->aPol,&pStyDoc->aNtm,&pStyDoc->aPrd) )
       {
-        SD_load_cfg(pStyDoc);
+        if( bConcreteCfg )
+          SD_load_ccfg(pStyDoc);
+        else
+          SD_load_cfg(pStyDoc);
         return SD_load_pat(pStyDoc);
       }
     }
@@ -1668,6 +1824,7 @@ static void SD_unload(StyDoc_T pStyDoc)
   MAP_freeMap(pStyDoc->pGMap);
   MAP_freeMap(pStyDoc->pTMap);
   MAP_freeMap(pStyDoc->pNTMap);
+  MAP_freeMap(pStyDoc->pClsMap);
   HS_DROP_SET(pStyDoc->pLSet);
   for( lI=0; lI < lCnt; ++lI )
     OT_DEL_T(OT_GET(ROW(_),pStyDoc->pDocBase,2*lI+1));
@@ -1677,7 +1834,7 @@ static void SD_unload(StyDoc_T pStyDoc)
   FreeMem(pStyDoc);
 }
 
-/* ------------------------- Main --------------------------------------- */
+/* ------------------------- Main ----------------------------------------- */
 
 int main(int argc, c_string argv[])
 { StyDoc_T pStyDoc;
